package com.example.tdl.domain.dto;

import java.util.HashMap;
import java.util.Map;


public class Result {
    public Integer code;
    public String message;

    /* OK */
    public final static Integer OK = 0x0001;
    public final static Integer SUCCESS = 0x0002;

    /* Success messages */
    public static final String STATUS_OK = "OK";
    public static final String STATUS_SUCCESS = "Success";
    public static final String DEVICE_CONTROLLED = "Device control successfully";
    public static final String DEVICE_CONFIGURED = "Device configure successfully";

    /* Error*/
    public final static Integer ERROR = 0xFFFF;

    /* Error Codes */
    public final static Integer ERR_TOPIC_SUBSCRIBE = 0x0101;
    public final static Integer ERR_TOPIC_UNSUBSCRIBE = 0x0102;
    public final static Integer ERR_DEVICE_MOUNT = 0x0103;
    public final static Integer ERR_DEVICE_UNMOUNT = 0x0104;
    public final static Integer ERR_NO_SENSOR_IN_GATEWAY = 0x0105;
    public final static Integer ERR_NO_SUCH_DEVICE = 0x0106;
    public final static Integer ERR_PARSE_DEVICE_CONFIG = 0x0107;
    public final static Integer ERR_TOPIC_PUBLISH = 0x0108;
    public final static Integer ERR_INVALID_TOPIC = 0x0109;
    public final static Integer ERR_DEVICE_OCCUPIED = 0x010A;
    public final static Integer ERR_DEVICE_TIMEOUT = 0x010B;

    /* Task Error Codes */
    public final static Integer ERR_TASK_TIMEOUT = 0x0110;
    public final static Integer ERR_SEM_ACQUIRE = 0x0111;

    /* Error messages */
    public static final String STATUS_ERROR = "Error";
    public static final String JSON_INVALID = "Json is invalid";
    public static final String CTRL_MSG_INVALID = "Control message is invalid";

    private final static Map<Integer, String> resultMap = new HashMap<Integer, String>() {
        {
            /* OK */
            put(OK, STATUS_OK);
            put(SUCCESS, STATUS_SUCCESS);
            /* Error */
            put(ERROR, STATUS_ERROR);
            /* Error Codes */
            put(ERR_TOPIC_SUBSCRIBE, "Topic subscribe error!");
            put(ERR_TOPIC_UNSUBSCRIBE, "Topic unsubscribe error!");
            put(ERR_TOPIC_PUBLISH, "Topic publish error!");
            put(ERR_DEVICE_MOUNT, "Device mount error!");
            put(ERR_DEVICE_UNMOUNT, "Device unmount error!");
            put(ERR_NO_SENSOR_IN_GATEWAY, "No such sensor in the gateway!");
            put(ERR_NO_SUCH_DEVICE, "No such device mounted!");
            put(ERR_PARSE_DEVICE_CONFIG, "Parse device config error!");
            put(ERR_INVALID_TOPIC, "Invalid topic name!");
            put(ERR_DEVICE_OCCUPIED, "Device is being occupied!");
            put(ERR_DEVICE_TIMEOUT, "Device response timeout!");
            /* Task Error Codes */
            put(ERR_TASK_TIMEOUT, "Task run timeout!");
            put(ERR_SEM_ACQUIRE, "Semaphore acquire error!");
        }
    };

    public Result(Integer code) {
        if (resultMap.containsKey(code)) {
            this.code = code;
            this.message = resultMap.get(code);
        } else {
            this.code = ERROR;
            this.message = resultMap.get(ERROR);
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Result) {
            return this.code.equals(((Result)obj).code);
        } else {
            return super.equals(obj);
        }
    }

    public boolean equals(Integer code) {
        return this.code.equals(code);
    }
}

